@@ -65,17 +65,19 @@ def create_parser():
     parser.add_argument('-o', '--outfile',
                         help='Output file name for corrected time-series')
 
-    # temporal functions
-    parser.add_argument('-t', '--template', dest='template_file',
-                        help='template file with the options')
-    parser.add_argument('--ex', '--exclude', dest='excludeDate', nargs='*', default=[],
-                        help='Exclude date(s) for DEM error estimation.\n' +
-                             'All dates will be corrected for DEM residual phase still.')
-    parser.add_argument('-p', '--poly-order', dest='polyOrder', type=int, default=2,
-                        help='polynomial order number of temporal deformation model, default = 2')
-    parser.add_argument('-s', '--step-date', dest='stepFuncDate', nargs='*', default=[],
-                        help='Date of step jump for temporal deformation model,'+
-                             ' i.e. date of earthquake/volcanic eruption')
+    defo_model = parser.add_argument_group('temporal deformation model')
+    defo_model.add_argument('-t', '--template', dest='template_file',
+                            help='template file with the options')
+    defo_model.add_argument('--ex', '--exclude', dest='excludeDate', nargs='*', default=[],
+                            help='Exclude date(s) for DEM error estimation.\n' +
+                                 'All dates will be corrected for DEM residual phase still.')
+    defo_model.add_argument('-p', '--poly-order', dest='polyOrder', type=int, default=2,
+                            help='polynomial order number of temporal deformation model (default: %(default)s).')
+    defo_model.add_argument('-s', '--step-date', dest='stepFuncDate', nargs='*', default=[],
+                            help='Date of step jump for temporal deformation model (default: %(default)s).'+
+                                 ' i.e. date of earthquake/volcanic eruption')
+    defo_model.add_argument('--periodic', '--period', '--peri', dest='periodic', type=float, nargs='+', default=[],
+                            help='periodic functinos of temporal deformation model (default: %(default)s).')
 
     parser.add_argument('--phase-velocity', dest='phaseVelocity', action='store_true',
                         help='Use phase velocity instead of phase for inversion constrain.')
@@ -158,7 +160,7 @@ def run_or_skip(inps):
         meta = readfile.read_attribute(inps.outfile)
         if any(str(vars(inps)[key]) != meta.get(key_prefix+key, 'None') for key in configKeys):
             flag = 'run'
-            print('3) NOT all key configration parameters are the same:{}'.format(configKeys))
+            print('3) NOT all key configuration parameters are the same:{}'.format(configKeys))
         else:
             print('3) all key configuration parameters are the same:{}'.format(configKeys))
 
@@ -187,7 +189,8 @@ def read_template2inps(template_file, inps=None):
             if key in ['polyOrder']:
                 iDict[key] = int(value)
             elif key in ['excludeDate','stepFuncDate']:
-                iDict[key] = ptime.yyyymmdd(value.replace(',', ' ').split())
+                value = value.replace('[','').replace(']','').replace(',', ' ')
+                iDict[key] = ptime.yyyymmdd(value.split())
 
     # computing configurations
     dask_key_prefix = 'mintpy.compute.'
@@ -239,6 +242,8 @@ def get_design_matrix4defo(inps):
     msg += "\ntemporal deformation model: polynomial order = {}".format(inps.polyOrder)
     if inps.stepFuncDate:
         msg += "\ntemporal deformation model: step functions at {}".format(inps.stepFuncDate)
+    if inps.periodic:
+        msg += "\ntemporal deformation model: periodic functions of {} yr".format(inps.periodic)
     msg += '\n'+'-'*80
     print(msg)
 
@@ -246,6 +251,7 @@ def get_design_matrix4defo(inps):
     model = dict()
     model['polynomial'] = inps.polyOrder
     model['step'] = inps.stepFuncDate
+    model['periodic'] = inps.periodic
     date_list = timeseries(inps.timeseries_file).get_date_list()
     G_defo = timeseries.get_design_matrix4time_func(date_list, model)
 
@@ -511,6 +517,10 @@ def correct_dem_error(inps):
 
     start_time = time.time()
 
+    # limit the number of threads to 1
+    # for slight speedup and big CPU usage save
+    num_threads_dict = cluster.set_num_threads("1")
+
     ## 1. input info
 
     # 1.1 read date info
@@ -647,6 +657,9 @@ def correct_dem_error(inps):
                                    datasetName='timeseries',
                                    block=block)
 
+    # roll back to the origial number of threads
+    cluster.roll_back_num_threads(num_threads_dict)
+
     # time info
     m, s = divmod(time.time()-start_time, 60)
     print('time used: {:02.0f} mins {:02.1f} secs.'.format(m, s))
