@@ -189,6 +189,9 @@ def cmd_line_parse(iargs=None):
         elif inps.obsDatasetName.startswith('rangeOffset'):
             inps.outfile = ['timeseriesRg.h5', 'residualInvRg.h5', 'numInvOffset.h5']
 
+        elif inps.obsDatasetName.startswith('ion'):
+            inps.outfile = ['timeseriesIon.h5', 'temporalCoherenceIon.h5', 'numInvIon.h5']
+
         else:
             raise ValueError('un-recognized input observation dataset name: {}'.format(inps.obsDatasetName))
 
@@ -277,7 +280,7 @@ def run_or_skip(inps):
 
         if any(str(vars(inps)[key]) != atr_ts.get(key_prefix+key, 'None') for key in configKeys):
             flag = 'run'
-            print('3) NOT all key configration parameters are the same: {}'.format(configKeys))
+            print('3) NOT all key configuration parameters are the same: {}'.format(configKeys))
         elif meta_keys and any(atr_ts[key] != atr_ifg[key] for key in meta_keys):
             flag = 'run'
             print('3) NOT all the metadata are the same: {}'.format(meta_keys))
@@ -297,7 +300,7 @@ def estimate_timeseries(A, B, tbase_diff, ifgram, weight_sqrt=None, min_norm_vel
 
     opt 1: X = np.dot(np.dot(numpy.linalg.inv(np.dot(B.T, B)), B.T), ifgram)
     opt 2: X = np.dot(numpy.linalg.pinv(B), ifgram)
-    opt 3: X = np.dot(scipy.linalg.pinv2(B), ifgram)
+    opt 3: X = np.dot(scipy.linalg.pinv(B), ifgram)
     opt 4: X = scipy.linalg.lstsq(B, ifgram)[0] [recommend and used]
 
     opt 4 supports weight.
@@ -335,7 +338,7 @@ def estimate_timeseries(A, B, tbase_diff, ifgram, weight_sqrt=None, min_norm_vel
                 inv_quality_name  - str, inversion quality type/name
     Returns:    ts                - 2D np.array in size of (num_date, num_pixel), phase time-series
                 inv_quality       - 1D np.array in size of (num_pixel), temporal coherence (for phase) or residual (for offset)
-                num_inv_obs       - 1D np.array in size of (num_pixel), number of observatons (ifgrams / offsets)
+                num_inv_obs       - 1D np.array in size of (num_pixel), number of observations (ifgrams / offsets)
                                     used during the inversion
     """
     ifgram = ifgram.reshape(A.shape[0], -1)
@@ -445,7 +448,7 @@ def calc_inv_quality(ifgram, G, X, inv_quality_name='temporalCoherence'):
     if num_pixel > chunk_size:
         num_chunk = int(np.ceil(num_pixel / chunk_size))
         num_chunk_step = max(1, int(ut.round_to_1(num_chunk / 5)))
-        print('calcualting {} in chunks of {} pixels: {} chunks in total ...'.format(
+        print('calculating {} in chunks of {} pixels: {} chunks in total ...'.format(
             inv_quality_name, chunk_size, num_chunk))
 
         for i in range(num_chunk):
@@ -787,7 +790,7 @@ def ifgram_inversion_patch(ifgram_file, box=None, ref_phase=None, obs_ds_name='u
     #time_idx = [i for i in range(num_date)]
     #time_idx.remove(ref_idx)
 
-    # 1.1 read / calcualte weight
+    # 1.1 read / calculate weight
     if weight_func in ['no', 'sbas']:
         weight = None
     else:
@@ -987,6 +990,26 @@ def ifgram_inversion(inps=None):
         inps = cmd_line_parse()
     start_time = time.time()
 
+    ## limit the number of threads in numpy/scipy to 1
+    #   and save the original value for roll back afterwards
+    #   becuase it does not increase the speed much but does increase the CPU usage significantly
+    #   as shown in the test note below.
+    # Dataset: SanFranSenDT42 version 1.x, patch 1 (505 x 510 x 1021) only
+    # Machine 1: Mac (6 Intel i7 CPUs/cores in 2.6 GHz)
+    # | dask (worker) | OMP_NUM_THREADS | Time used (sec) | CPU usage |
+    # |   no   (0)    |        4        |      850        | 1 x 300%  |
+    # |   no   (0)    |        1        |      930        | 1 x 100%  |
+    # | local  (4)    |        4        |      580        | 4 x 250%  |
+    # | local  (4)    |        1        |      420        | 4 x 100%  |
+    # Machine 2: Linux local cluster (16 Intel E5 CPUs/cores in 2.4 GHz)
+    # | dask (worker) | OMP_NUM_THREADS | Time used (sec) | CPU usage |
+    # |   no   (0)    |        4        |     1400        | 1 x 400%  |
+    # |   no   (0)    |        1        |     1250        | 1 x 100%  |
+    # | local  (4)    |        4        |      750        | 4 x 320%  |
+    # | local  (4)    |        1        |      500        | 4 x 100%  |
+    num_threads_dict = cluster.set_num_threads("1")
+
+
     ## 1. input info
 
     stack_obj = ifgramStack(inps.ifgramStackFile)
@@ -1055,7 +1078,7 @@ def ifgram_inversion(inps=None):
         "bperp"      : [np.float32,  (num_date,), pbase],
         "timeseries" : [np.float32,  (num_date, length, width), None],
     }
-    writefile.layout_hdf5(inps.tsFile, ds_name_dict, meta)
+    writefile.layout_hdf5(inps.tsFile, ds_name_dict, metadata=meta)
 
     # 2.3 instantiate invQualifyFile: temporalCoherence / residualInv
     if 'residual' in os.path.basename(inps.invQualityFile).lower():
@@ -1175,6 +1198,9 @@ def ifgram_inversion(inps=None):
         with h5py.File(inps.numInvFile, 'r+') as f:
             f['mask'][ref_y, ref_x] = num_ifgram
 
+    # roll back to the original number of threads
+    cluster.roll_back_num_threads(num_threads_dict)
+
     m, s = divmod(time.time() - start_time, 60)
     print('time used: {:02.0f} mins {:02.1f} secs.\n'.format(m, s))
     return
