@@ -12,22 +12,27 @@ import glob
 import time
 import argparse
 import warnings
+
 from mintpy.defaults import auto_path
 from mintpy.defaults.template import get_template_content
-from mintpy.objects import (geometryDatasetNames,
-                            geometry,
-                            ifgramDatasetNames,
-                            ifgramStack,
-                            sensor)
-from mintpy.objects.stackDict import (geometryDict,
-                                      ifgramStackDict,
-                                      ifgramDict)
+from mintpy.objects import (
+    geometryDatasetNames,
+    geometry,
+    ifgramDatasetNames,
+    ifgramStack,
+    sensor,
+)
+from mintpy.objects.stackDict import (
+    geometryDict,
+    ifgramStackDict,
+    ifgramDict,
+)
 from mintpy.utils import readfile, ptime, utils as ut
 from mintpy import subset
 
 
 #################################################################
-PROCESSOR_LIST = ['isce', 'aria', 'snap', 'gamma', 'roipac']
+PROCESSOR_LIST = ['isce', 'aria', 'hyp3', 'gmtsar', 'snap', 'gamma', 'roipac']
 
 datasetName2templateKey = {
     'unwrapPhase'     : 'mintpy.load.unwFile',
@@ -142,7 +147,7 @@ def read_inps2dict(inps):
     It grab the following contents into iDict
     1. inps & all template files
     2. configurations: processor, autoPath, updateMode, compression, x/ystep
-    3. extra metadata: PLATFORM, PROJECT_NAME, 
+    3. extra metadata: PLATFORM, PROJECT_NAME,
     4. translate autoPath
     """
     # Read input info into iDict
@@ -422,43 +427,65 @@ def read_inps_dict2ifgram_stack_dict_object(iDict):
 
     # dsPathDict --> pairsDict --> stackObj
     dsNameList = list(dsPathDict.keys())
+
+    #####################################
+    # A dictionary of data file paths for a list of pairs, e.g.:
+    # pairsDict = {
+    #     ('date1', 'date2') : ifgramPathDict1,
+    #     ('date1', 'date3') : ifgramPathDict2,
+    #     ...,
+    # }
+
     pairsDict = {}
-    for dsPath in dsPathDict[dsName0]:
+    for i, dsPath0 in enumerate(dsPathDict[dsName0]):
         # date string used in the file/dir path
+        # YYYYDDD       for gmtsar [modern Julian date]
         # YYYYMMDDTHHMM for uavsar
-        # YYYYMMDD for all the others
-        date12 = readfile.read_attribute(dsPath)['DATE12'].replace('_','-')
-        dates = ptime.yyyymmdd(date12.split('-'))
+        # YYYYMMDD      for all the others
+        date6s = readfile.read_attribute(dsPath0)['DATE12'].replace('_','-').split('-')
+        if iDict['processor'] == 'gmtsar':
+            date12MJD = os.path.basename(os.path.dirname(dsPath0))
+        else:
+            date12MJD = None
 
         #####################################
-        # A dictionary of data files for a given pair.
-        # One pair may have several types of dataset.
-        # example ifgramPathDict = {'unwrapPhase': /pathToFile/filt.unw,
-        #                           'ionoPhase'  : /PathToFile/iono.bil}
-        # All path of data file must contain the reference and secondary date, either in file name or folder name.
+        # A dictionary of data file paths for a given pair.
+        # One pair may have several types of dataset, e.g.:
+        # ifgramPathDict1 = {
+        #     'unwrapPhase': /dirPathToFile/filt_fine.unw,
+        #     'coherence'  : /dirPathToFile/filt_fine.cor,
+        #     'ionoPhase'  : /dirPathToFile/iono.bil,
+        #     ...
+        # }
+        # All path of data file must contain the reference and secondary date, in file/dir name.
 
         ifgramPathDict = {}
-        for i in range(len(dsNameList)):
-            dsName = dsNameList[i]
-            dsPath1 = dsPathDict[dsName][0]
-
-            if all(d[2:] in dsPath1 for d in dates):
+        for dsName in dsNameList:
+            # search the matching data file for the given date12
+            # 1st guess: file in the same order as the one for dsName0
+            dsPath1 = dsPathDict[dsName][i]
+            if (all(d6 in dsPath1 for d6 in date6s)
+                    or (date12MJD and date12MJD in dsPath1)):
                 ifgramPathDict[dsName] = dsPath1
 
             else:
-                dsPath2 = [i for i in dsPathDict[dsName]
-                           if all(d[2:] in i for d in dates)]
+                # 2nd guess: any file in the list
+                dsPath2 = [p for p in dsPathDict[dsName]
+                           if (all(d6 in p for d6 in date6s)
+                                   or (date12MJD and date12MJD in dsPath1))]
 
                 if len(dsPath2) > 0:
                     ifgramPathDict[dsName] = dsPath2[0]
+
                 else:
-                    print('WARNING: {} file missing for pair {}'.format(dsName, dates))
+                    print('WARNING: {:>18} file missing for pair {}'.format(dsName, date6s))
 
         # initiate ifgramDict object
         ifgramObj = ifgramDict(datasetDict=ifgramPathDict)
 
         # update pairsDict object
-        pairsDict[tuple(dates)] = ifgramObj
+        date8s = ptime.yyyymmdd(date6s)
+        pairsDict[tuple(date8s)] = ifgramObj
 
     if len(pairsDict) > 0:
         stackObj = ifgramStackDict(pairsDict=pairsDict, dsName0=dsName0)
@@ -469,15 +496,18 @@ def read_inps_dict2ifgram_stack_dict_object(iDict):
 
 def read_inps_dict2geometry_dict_object(iDict):
 
-    # eliminate dsName by processor
+    # eliminate lookup table dsName for input files in radar-coordinates
     if iDict['processor'] in ['isce', 'doris']:
+        # for processors with lookup table in radar-coordinates, remove azimuth/rangeCoord
         iDict['ds_name2key'].pop('azimuthCoord')
         iDict['ds_name2key'].pop('rangeCoord')
     elif iDict['processor'] in ['roipac', 'gamma']:
+        # for processors with lookup table in geo-coordinates, remove latitude/longitude
         iDict['ds_name2key'].pop('latitude')
         iDict['ds_name2key'].pop('longitude')
-    elif iDict['processor'] in ['snap']:
-        #check again when there is a SNAP product in radar coordiantes
+    elif iDict['processor'] in ['aria', 'gmtsar', 'hyp3', 'snap']:
+        # for processors with geocoded products support only, do nothing for now.
+        # check again when adding products support in radar-coordiantes
         pass
     else:
         print('Un-recognized InSAR processor: {}'.format(iDict['processor']))
@@ -516,14 +546,17 @@ def read_inps_dict2geometry_dict_object(iDict):
         print('WARNING: No reqired {} data files found!'.format(dsName0))
 
     # metadata
-    ifgramRadarMetadata = None
+    ifgramMetaGeo = None
+    ifgramMetaRadar = None
     ifgramKey = iDict['ds_name2key']['unwrapPhase']
     if ifgramKey in iDict.keys():
         ifgramFiles = glob.glob(str(iDict[ifgramKey]))
         if len(ifgramFiles) > 0:
             atr = readfile.read_attribute(ifgramFiles[0])
-            if 'Y_FIRST' not in atr.keys():
-                ifgramRadarMetadata = atr.copy()
+            if 'Y_FIRST' in atr.keys():
+                ifgramMetaGeo = atr.copy()
+            else:
+                ifgramMetaRadar = atr.copy()
 
     # dsPathDict --> dsGeoPathDict + dsRadarPathDict
     dsNameList = list(dsPathDict.keys())
@@ -544,11 +577,11 @@ def read_inps_dict2geometry_dict_object(iDict):
     if len(dsRadarPathDict) > 0:
         geomRadarObj = geometryDict(processor=iDict['processor'],
                                     datasetDict=dsRadarPathDict,
-                                    extraMetadata=ifgramRadarMetadata)
+                                    extraMetadata=ifgramMetaRadar)
     if len(dsGeoPathDict) > 0:
         geomGeoObj = geometryDict(processor=iDict['processor'],
                                   datasetDict=dsGeoPathDict,
-                                  extraMetadata=None)
+                                  extraMetadata=ifgramMetaGeo)
     return geomRadarObj, geomGeoObj
 
 
@@ -594,10 +627,12 @@ def prepare_metadata(iDict):
     print('-'*50)
     print('prepare metadata files for {} products'.format(processor))
 
-    if processor in ['gamma', 'roipac', 'snap']:
+    if processor in ['gamma', 'hyp3', 'roipac', 'snap']:
         # import prep_module
         if processor == 'gamma':
             from mintpy import prep_gamma as prep_module
+        elif processor == 'hyp3':
+            from mintpy import prep_hyp3 as prep_module
         elif processor == 'roipac':
             from mintpy import prep_roipac as prep_module
         elif processor == 'snap':
@@ -646,12 +681,20 @@ def prepare_metadata(iDict):
             obs_dir = None
             obs_file = None
 
+        # geometry
+        geom_names = ['dem', 'lookupY', 'lookupX', 'incAngle', 'azAngle', 'shadowMask', 'waterMask']
+        geom_keys = ['mintpy.load.{}File'.format(i) for i in geom_names]
+        geom_files = [os.path.basename(iDict[key]) for key in geom_keys
+                      if (iDict[key] and iDict[key].lower() != 'auto')]
+
         # compose list of input arguments
         iargs = ['-m', meta_file, '-g', geom_dir]
         if baseline_dir:
             iargs += ['-b', baseline_dir]
         if obs_dir is not None:
             iargs += ['-d', obs_dir, '-f', obs_file]
+        if geom_files:
+            iargs += ['--geom-files'] + geom_files
 
         # run module
         print('prep_isce.py', ' '.join(iargs))
@@ -664,7 +707,7 @@ def prepare_metadata(iDict):
         from mintpy import prep_aria
 
         ## compose input arguments
-        # use the default template file is exists & input
+        # use the default template file if exists & input
         default_temp_files = [fname for fname in iDict['template_file'] if fname.endswith('smallbaselineApp.cfg')]
         if len(default_temp_files) > 0:
             temp_file = default_temp_files[0]
@@ -707,6 +750,22 @@ def prepare_metadata(iDict):
         except:
             warnings.warn('prep_aria.py failed. Assuming its result exists and continue...')
 
+    elif processor == 'gmtsar':
+        from mintpy import prep_gmtsar
+
+        # use the custom template file if exists & input
+        custom_temp_files = [fname for fname in iDict['template_file'] if not fname.endswith('smallbaselineApp.cfg')]
+        if len(custom_temp_files) == 0:
+            raise FileExistsError('Custom template file NOT found and is required for GMTSAR!')
+
+        # run prep_*.py
+        iargs = [custom_temp_files[0], '--mintpy-dir', os.path.dirname(iDict['outdir'])]
+        print('prep_gmtsar.py', ' '.join(iargs))
+        try:
+            prep_gmtsar.main(iargs)
+        except:
+            warnings.warn('prep_gmtsar.py failed. Assuming its result exists and continue...')
+
     else:
         msg = 'un-recognized InSAR processor: {}'.format(processor)
         msg += '\nsupported processors: {}'.format(PROCESSOR_LIST)
