@@ -368,15 +368,21 @@ class geometryDict:
             return None
 
         if 'Y_FIRST' in self.extraMetadata.keys():
-            # for dataset in geo-coordinates, use contant value from SLANT_RANGE_DISTANCE.
+            # for dataset in geo-coordinates, use:
+            # 1) incidenceAngle matrix if available OR
+            # 2) contant value from SLANT_RANGE_DISTANCE.
+            ds_name = 'incidenceAngle'
             key = 'SLANT_RANGE_DISTANCE'
-            print('geocoded input, use contant value from metadata {}'.format(key))
-            if key in self.extraMetadata.keys():
+            if ds_name in self.dsNames:
+                print('geocoded input, use incidenceAngle from file {}'.format(self.datasetDict[ds_name]))
+                inc_angle = self.read(family=ds_name)[0].astype(np.float32)
+                data = ut.incidence_angle2slant_range_distance(self.extraMetadata, inc_angle)
+            elif key in self.extraMetadata.keys():
+                print('geocoded input, use contant value from metadata {}'.format(key))
                 length = int(self.extraMetadata['LENGTH'])
                 width = int(self.extraMetadata['WIDTH'])
                 range_dist = float(self.extraMetadata[key])
                 data = np.ones((length, width), dtype=np.float32) * range_dist
-
             else:
                 return None
 
@@ -586,13 +592,13 @@ class geometryDict:
                                               xstep=xstep,
                                               ystep=ystep)[0], dtype=dsDataType)
 
-                    # water body: -1 for water and 0 for land
-                    # water mask:  0 for water and 1 for land
+                    # water body: -1/True  for water and 0/False for land
+                    # water mask:  0/False for water and 1/True  for land
                     fname = os.path.basename(self.datasetDict[dsName])
                     if fname.startswith('waterBody') or fname.endswith('.wbd'):
-                        data = data > -0.5
-                        print(('    input file "{}" is water body (-1/0 for water/land), '
-                               'convert to water mask (0/1 for water/land).'.format(fname)))
+                        data = ~data
+                        print(('    input file "{}" is water body (True/False for water/land), '
+                               'convert to water mask (False/True for water/land).'.format(fname)))
 
                     elif dsName == 'height':
                         noDataValueDEM = -32768
@@ -608,6 +614,13 @@ class geometryDict:
                         print('    scale value of {:<15} by 1/{} due to multilooking'.format(dsName, ystep))
                         data /= ystep
 
+                    elif dsName == 'incidenceAngle':
+                        atr = readfile.read_attribute(self.file)
+                        if (atr.get('PROCESSOR', 'isce') == 'hyp3'
+                                and atr.get('UNIT', 'degrees').startswith('rad')):
+                            print('    convert the unit of {:<15} from radian to degree'.format(dsName))
+                            data *= 180. / np.pi
+
                     # write
                     ds = f.create_dataset(dsName,
                                           data=data,
