@@ -32,8 +32,8 @@ REFERENCE = """references:
   Parameters and Uncertainties: A Bayesian Approach, Geochemistry, Geophysics, Geosystems, 19, 
   doi:10.1029/2018GC007585.
 
-  Yunjun, Z., F. Amelung, Y. Aoki, (2021), Imaging the Hydrothermal System of Kirishima Volcanic Complex, 
-  Japan from ALOS-1/2 InSAR time-series, (in prep.).
+  Yunjun, Z., Amelung, F., & Aoki, Y. (2021), Imaging the hydrothermal system of Kirishima volcanic complex 
+  with L-band InSAR time series, Geophysical Research Letters, 48(11), e2021GL092879. doi:10.1029/2021GL092879
 """
 
 def create_parser():
@@ -55,9 +55,9 @@ def create_parser():
     parser.add_argument('--out-dir', dest='outdir',
                         help='custom output directory, ONLY IF --output is not specified.')
     parser.add_argument('--ellipsoid2geoid', action='store_true',
-                        help='Convert the height of ellipsoid to geoid using geoidheight module\n'+
-                             'Download/install geoidheight as below:\n'+
-                             'https://github.com/geodesymiami/Yunjun_et_al-2019-Kirishima/GBIS')
+                        help='Convert the height of ellipsoid to geoid using "geoidheight" module\n'+
+                             'Download & install geoidheight as below:\n'+
+                             'https://github.com/geodesymiami/2021_Kirishima')
     return parser
 
 
@@ -103,8 +103,6 @@ def read_data(inps):
 
         # update mask to exclude pixel with NaN value
         inps.mask *= ~np.isnan(inps.phase)
-        # set all masked out pixel to NaN
-        inps.phase[inps.mask==0] = np.nan
     else:
         raise ValueError("input file not support yet: {}".format(k))
     print('number of pixels: {}'.format(np.sum(inps.mask)))
@@ -122,6 +120,13 @@ def read_data(inps):
         inps.metadata['REF_Y'] = ref_y
         inps.metadata['REF_X'] = ref_x
 
+    # mask out pixels with zero phase value
+    ref_y = int(inps.metadata['REF_Y'])
+    ref_x = int(inps.metadata['REF_X'])
+    inps.mask *= inps.phase != 0
+    inps.mask[ref_y, ref_x] = 1
+    print('number of pixels after excluding zero phase value: {}'.format(np.sum(inps.mask)))
+
     # read geometry
     inps.lat, inps.lon = ut.get_lat_lon(inps.metadata)
     inps.inc_angle = readfile.read(inps.geom_file, datasetName='incidenceAngle')[0]
@@ -148,6 +153,8 @@ def read_data(inps):
         msg += '\n\tby subtracting a constant offset of {:.2f} m'.format(h_offset)
         print(msg)
 
+    # masking
+    inps.phase[inps.mask==0] = np.nan
     inps.lat[inps.mask==0] = np.nan
     inps.lon[inps.mask==0] = np.nan
     inps.inc_angle[inps.mask==0] = np.nan
@@ -176,7 +183,7 @@ def plot_data(inps):
     defo = inps.phase / inps.range2phase * 100. #convert to deformation in cm
     dmin, dmax = np.nanmin(defo), np.nanmax(defo)
     dlim = max(abs(dmin), abs(dmax))
-    im = axs[0].imshow(defo, vmin=-dlim, vmax=dlim, cmap='jet');
+    im = axs[0].imshow(defo, vmin=-dlim, vmax=dlim, cmap='jet', interpolation='nearest')
     # reference point
     axs[0].plot(int(inps.metadata['REF_X']),
                 int(inps.metadata['REF_Y']), 'ks', ms=6)
@@ -189,7 +196,7 @@ def plot_data(inps):
     for ax, data, title in zip(axs[1:],
                                [inps.lat, inps.lon, inps.inc_angle, inps.head_angle, inps.height],
                                ['Latitude', 'Longitude', 'Incidence Angle', 'Head Angle', 'Height']):
-        im = ax.imshow(data, cmap='jet')
+        im = ax.imshow(data, cmap='jet', interpolation='nearest')
         ax.set_title(title)
         cbar = fig.colorbar(im, ax=ax)
         if title == 'Height':
