@@ -82,7 +82,7 @@ def create_parser():
 
     parser.add_argument('-o', '--output', dest='outfile',
                         help='output file name\n' +
-                             'add prefix "subset_" if input/output files are in the same directory;\n' +
+                             'add prefix "sub_" if input/output files are in the same directory;\n' +
                              'same filename otherwise.')
 
     dset_group = parser.add_argument_group('Datasets',
@@ -148,20 +148,25 @@ def read_subset_template2box(template_file):
     Return None if not specified.
     """
     tmpl = readfile.read_template(template_file)
+
+    # subset.lalo -> geo_box
     try:
-        sub = [i.strip() for i in tmpl['mintpy.subset.lalo'].split(',')]
-        sub_lat = sorted([float(i.strip()) for i in sub[0].split(':')])
-        sub_lon = sorted([float(i.strip()) for i in sub[1].split(':')])
-        geo_box = (sub_lon[0], sub_lat[1], sub_lon[1], sub_lat[0])
+        opts = [i.strip().replace('[','').replace(']','') for i in tmpl['mintpy.subset.lalo'].split(',')]
+        lat0, lat1 = sorted([float(i.strip()) for i in opts[0].split(':')])
+        lon0, lon1 = sorted([float(i.strip()) for i in opts[1].split(':')])
+        geo_box = (lon0, lat1, lon1, lat0)
     except:
         geo_box = None
+
+    # subset.yx -> pix_box
     try:
-        sub = [i.strip() for i in tmpl['mintpy.subset.yx'].split(',')]
-        sub_y = sorted([int(i.strip()) for i in sub[0].split(':')])
-        sub_x = sorted([int(i.strip()) for i in sub[1].split(':')])
-        pix_box = (sub_x[0], sub_y[0], sub_x[1], sub_y[1])
+        opts = [i.strip().replace('[','').replace(']','') for i in tmpl['mintpy.subset.yx'].split(',')]
+        y0, y1 = sorted([int(i.strip()) for i in opts[0].split(':')])
+        x0, x1 = sorted([int(i.strip()) for i in opts[1].split(':')])
+        pix_box = (x0, y0, x1, y1)
     except:
         pix_box = None
+
     return pix_box, geo_box
 
 
@@ -256,14 +261,14 @@ def subset_input_dict2box(subset_dict, meta_dict):
 
     # Use subset_lat/lon input if existed,  priority: lat/lon > y/x > len/wid
     coord = ut.coordinate(meta_dict)
-    if subset_dict['subset_lat']:
+    if subset_dict.get('subset_lat', None):
         sub_y = coord.lalo2yx(subset_dict['subset_lat'], coord_type='latitude')
     elif subset_dict['subset_y']:
         sub_y = subset_dict['subset_y']
     else:
         sub_y = [0, length]
 
-    if subset_dict['subset_lon']:
+    if subset_dict.get('subset_lon', None):
         sub_x = coord.lalo2yx(subset_dict['subset_lon'], coord_type='longitude')
     elif subset_dict['subset_x']:
         sub_x = subset_dict['subset_x']
@@ -380,7 +385,7 @@ def subset_file(fname, subset_dict_input, out_file=None):
                 out_file = '{}_tight{}'.format(os.path.splitext(fname)[0],
                                                os.path.splitext(fname)[1])
             else:
-                out_file = 'subset_'+os.path.basename(fname)
+                out_file = 'sub_'+os.path.basename(fname)
         else:
             out_file = os.path.basename(fname)
     print('writing >>> '+out_file)
@@ -455,7 +460,7 @@ def subset_file(fname, subset_dict_input, out_file=None):
 
                         prog_bar.update(i+1, suffix='{}/{}'.format(i+1, ds_shape[0]))
                     prog_bar.close()
-                    print('finished writing to file: {}'.format(fname))
+                    print('finished writing to file: {}'.format(out_file))
 
     else:
         # IO for binary files
@@ -472,6 +477,28 @@ def subset_file(fname, subset_dict_input, out_file=None):
                         metadata=atr,
                         ref_file=fname)
 
+
+        # write extra metadata files for ISCE data files
+        if os.path.isfile(fname+'.xml') or os.path.isfile(fname+'.aux.xml'):
+            # write ISCE XML file
+            dtype_gdal = readfile.NUMPY2GDAL_DATATYPE[atr['DATA_TYPE']]
+            dtype_isce = readfile.GDAL2ISCE_DATATYPE[dtype_gdal]
+            writefile.write_isce_xml(
+                out_file,
+                width=int(atr['WIDTH']),
+                length=int(atr['LENGTH']),
+                bands=len(dsDict.keys()),
+                data_type=dtype_isce,
+                scheme=atr['scheme'])
+            print(f'write file: {out_file}.xml')
+
+            # write GDAL VRT file
+            if os.path.isfile(fname+'.vrt'):
+                from isceobj.Util.ImageUtil import ImageLib as IML
+                img = IML.loadImage(out_file)[0]
+                img.renderVRT()
+                print(f'write file: {out_file}.vrt')
+
     return out_file
 
 
