@@ -62,9 +62,9 @@ REFERENCE = """reference:
   2324-2341, doi:10.1002/2013JB010588.
 
   # ERA-5
-  Copernicus Climate Change Service (2017): ERA5: Fifth generation of ECMWF atmospheric reanalyses of 
-  the global climate. Copernicus Climate Change Service Climate Data Store, 
-  date of access: {your_date_of_access}. https://cds.climate.copernicus.eu/cdsapp#!/hom
+  Hersbach, H., Bell, B., Berrisford, P., Hirahara, S., Horányi, A., Muñoz-Sabater, J., et al. (2020). 
+  The ERA5 global reanalysis. Quarterly Journal of the Royal Meteorological Society, 146(730), 1999–2049.
+  https://doi.org/10.1002/qj.3803
 """
 
 DATA_INFO = """Global Atmospheric Models:
@@ -263,7 +263,7 @@ def get_grib_info(inps):
 
     # area extent for ERA5 grib data download
     if inps.atr:
-        inps.snwe = get_snwe(inps.atr)
+        inps.snwe = get_snwe(inps.atr, geom_file=inps.geom_file)
     else:
         inps.snwe = None
 
@@ -388,9 +388,9 @@ def floor2multiple(x, step=10):
     return x - x % step
 
 
-def get_snwe(meta, min_buffer=2, step=10):
+def get_snwe(meta, geom_file=None, min_buffer=2, step=10):
     # get bounding box
-    lat0, lat1, lon0, lon1 = get_bounding_box(meta)
+    lat0, lat1, lon0, lon1 = get_bounding_box(meta, geom_file=geom_file)
 
     # lat/lon0/1 --> SNWE
     S = np.floor(min(lat0, lat1) - min_buffer).astype(int)
@@ -437,7 +437,7 @@ def snwe2str(snwe):
     return area
 
 
-def get_bounding_box(meta):
+def get_bounding_box(meta, geom_file=None):
     """Get lat/lon range (roughly), in the same order of data file
     lat0/lon0 - starting latitude/longitude (first row/column)
     lat1/lon1 - ending latitude/longitude (last row/column)
@@ -451,14 +451,39 @@ def get_bounding_box(meta):
         lon_step = float(meta['X_STEP'])
         lat1 = lat0 + lat_step * (length - 1)
         lon1 = lon0 + lon_step * (width - 1)
+
+        # 'Y_FIRST' not in 'degree'
+        # e.g. meters for UTM projection from ASF HyP3
+        y_unit = meta.get('Y_UNIT', 'degrees').lower()
+        if not y_unit.startswith('deg'):
+            lat0, lon0 = ut.to_latlon(meta['OG_FILE_PATH'], lon0, lat0)
+            lat1, lon1 = ut.to_latlon(meta['OG_FILE_PATH'], lon1, lat1)
+
     else:
         # radar coordinates
-        lats = [float(meta['LAT_REF{}'.format(i)]) for i in [1,2,3,4]]
-        lons = [float(meta['LON_REF{}'.format(i)]) for i in [1,2,3,4]]
-        lat0 = np.mean(lats[0:2])
-        lat1 = np.mean(lats[2:4])
-        lon0 = np.mean(lons[0:3:2])
-        lon1 = np.mean(lons[1:4:2])
+        if geom_file and os.path.isfile(geom_file):
+            geom_dset_list = readfile.get_dataset_list(geom_file)
+        else:
+            geom_dset_list = []
+
+        if 'latitude' in geom_dset_list:
+            lats = readfile.read(geom_file, datasetName='latitude')[0]
+            lons = readfile.read(geom_file, datasetName='longitude')[0]
+            lats[lats == 0] = np.nan
+            lons[lons == 0] = np.nan
+            lat0 = np.nanmin(lats)
+            lat1 = np.nanmax(lats)
+            lon0 = np.nanmin(lons)
+            lon1 = np.nanmax(lons)
+
+        else:
+            lats = [float(meta['LAT_REF{}'.format(i)]) for i in [1,2,3,4]]
+            lons = [float(meta['LON_REF{}'.format(i)]) for i in [1,2,3,4]]
+            lat0 = np.mean(lats[0:2])
+            lat1 = np.mean(lats[2:4])
+            lon0 = np.mean(lons[0:3:2])
+            lon1 = np.mean(lons[1:4:2])
+
     return lat0, lat1, lon0, lon1
 
 
@@ -508,13 +533,13 @@ def dload_grib_files(grib_files, tropo_model='ERA5', snwe=None):
     # Get date list to download (skip already downloaded files)
     grib_files_exist = check_exist_grib_file(grib_files, print_msg=True)
     grib_files2dload = sorted(list(set(grib_files) - set(grib_files_exist)))
-    date_list2dload = [str(re.findall('\d{8}', i)[0]) for i in grib_files2dload]
+    date_list2dload = [str(re.findall('\d{8}', os.path.basename(i))[0]) for i in grib_files2dload]
     print('number of grib files to download: %d' % len(date_list2dload))
     print('------------------------------------------------------------------------------\n')
 
     # Download grib file using PyAPS
     if len(date_list2dload) > 0:
-        hour = re.findall('\d{8}[-_]\d{2}', grib_files2dload[0])[0].replace('-', '_').split('_')[1]
+        hour = re.findall('\d{8}[-_]\d{2}', os.path.basename(grib_files2dload[0]))[0].replace('-', '_').split('_')[1]
         grib_dir = os.path.dirname(grib_files2dload[0])
 
         # try 3 times to download, then use whatever downloaded to calculate delay
@@ -547,7 +572,7 @@ def dload_grib_files(grib_files, tropo_model='ERA5', snwe=None):
     return grib_files
 
 
-def get_delay(grib_file, tropo_model, delay_type, dem, inc, lat, lon, verbose):
+def get_delay(grib_file, tropo_model, delay_type, dem, inc, lat, lon, mask=None, verbose=False):
     """Get delay matrix using PyAPS for one acquisition
     Parameters: grib_file       - str, grib file path
                 tropo_model     - str, GAM model
@@ -568,6 +593,7 @@ def get_delay(grib_file, tropo_model, delay_type, dem, inc, lat, lon, verbose):
                        inc=inc,
                        lat=lat,
                        lon=lon,
+                       mask=mask,
                        verb=verbose)
 
     # estimate delay
@@ -603,7 +629,7 @@ def calc_delay_timeseries(inps):
             print('1) output file exists and is newer than all GRIB files.')
 
             # check dataset size in space / time
-            date_list = [str(re.findall('\d{8}', i)[0]) for i in grib_files]
+            date_list = [str(re.findall('\d{8}', os.path.basename(i))[0]) for i in grib_files]
             if (get_dataset_size(tropo_file) != get_dataset_size(geom_file) 
                     or any(i not in timeseries(tropo_file).get_date_list() for i in date_list)):
                 flag = 'run'
@@ -647,10 +673,17 @@ def calc_delay_timeseries(inps):
         # for lookup table in geo-coded (gamma, roipac) and obs. in geo-coord
         inps.lat, inps.lon = ut.get_lat_lon(geom_obj.metadata)
 
+        # convert coordinates to lat/lon, e.g. from UTM for ASF HyPP3
+        if not geom_obj.metadata['Y_UNIT'].startswith('deg'):
+            inps.lat, inps.lon = ut.to_latlon(inps.atr['OG_FILE_PATH'], inps.lon, inps.lat)
+
     else:
         # for lookup table in geo-coded (gamma, roipac) and obs. in radar-coord
         inps.lat, inps.lon = ut.get_lat_lon_rdc(inps.atr)
 
+    # mask of valid pixels
+    mask = np.multiply(inps.inc != 0, ~np.isnan(inps.inc))
+
 
     ## 2. prepare output file
     # metadata
@@ -667,7 +700,7 @@ def calc_delay_timeseries(inps):
     # instantiate time-series
     length, width = int(atr['LENGTH']), int(atr['WIDTH'])
     num_date = len(inps.grib_files)
-    date_list = [str(re.findall('\d{8}', i)[0]) for i in inps.grib_files]
+    date_list = [str(re.findall('\d{8}', os.path.basename(i))[0]) for i in inps.grib_files]
     dates = np.array(date_list, dtype=np.string_)
     ds_name_dict = {
         "date"       : [dates.dtype, (num_date,), dates],
@@ -678,7 +711,7 @@ def calc_delay_timeseries(inps):
 
     ## 3. calculate phase delay
     print('\n------------------------------------------------------------------------------')
-    print('calcualting absolute delay for each date using PyAPS (Jolivet et al., 2011; 2014) ...')
+    print('calculating absolute delay for each date using PyAPS (Jolivet et al., 2011; 2014) ...')
     print('number of grib files used: {}'.format(num_date))
 
     prog_bar = ptime.progressBar(maxValue=num_date, print_msg=~inps.verbose)
@@ -693,6 +726,7 @@ def calc_delay_timeseries(inps):
                                inc=inps.inc,
                                lat=inps.lat,
                                lon=inps.lon,
+                               mask=mask,
                                verbose=inps.verbose)
 
         # write tropo delay to file
