@@ -10,7 +10,7 @@ import os
 import sys
 import re
 import time
-from datetime import datetime as dt, timedelta
+import datetime as dt
 import numpy as np
 from mintpy.objects.progress import progressBar
 
@@ -62,7 +62,7 @@ def round_seconds(datetime_obj):
     """
     datetime_obj_out = datetime_obj
     if datetime_obj_out.microsecond >= 5e5:
-        datetime_obj_out += timedelta(seconds=1)
+        datetime_obj_out += dt.timedelta(seconds=1)
     return datetime_obj_out.replace(microsecond=0)
 
 
@@ -74,7 +74,7 @@ def yyyymmdd2season(date_str):
     """
     # get day of the year
     date_str = yyyymmdd(date_str)
-    yday = dt.strptime(date_str, "%Y%m%d").timetuple().tm_yday
+    yday = dt.datetime.strptime(date_str, "%Y%m%d").timetuple().tm_yday
 
     # determine the season
     season = None
@@ -94,9 +94,9 @@ def datenum2datetime(datenum):
     Parameters: datenum : Date in datenum format, i.e. 731763.5
     Returns:    datetime: Date in datetime.datetime format, datetime.datetime(2003, 7, 1, 12, 0)
     """
-    return dt.fromordinal(int(datenum)) \
-           + timedelta(days=datenum % 1) \
-           - timedelta(days=366)
+    return dt.datetime.fromordinal(int(datenum)) \
+           + dt.timedelta(days=datenum % 1) \
+           - dt.timedelta(days=366)
 
 
 def decimal_year2datetime(years):
@@ -110,7 +110,7 @@ def decimal_year2datetime(years):
         yday = np.floor((x - year) * 365.25).astype(int) + 1
         x2 = '{:d}-{:d}'.format(year, yday)
         try:
-            xt = dt.strptime(x2, "%Y-%j")
+            xt = dt.datetime.strptime(x2, "%Y-%j")
         except:
             raise ValueError('wrong format: ',x)
         return xt
@@ -145,10 +145,10 @@ def yyyymmdd2years(dates):
 
     years = []
     for date_str in date_list:
-        d = dt.strptime(date_str, date_format)
-        y = (d.year + (d.timetuple().tm_yday - 1) / 365.25 + 
-             d.hour / (365.25 * 24) + 
-             d.minute / (365.25 * 24 * 60) + 
+        d = dt.datetime.strptime(date_str, date_format)
+        y = (d.year + (d.timetuple().tm_yday - 1) / 365.25 +
+             d.hour / (365.25 * 24) +
+             d.minute / (365.25 * 24 * 60) +
              d.second / (365.25 * 24 * 60 * 60))
         years.append(y)
 
@@ -177,9 +177,9 @@ def yy2yyyy(year):
 
 
 def yyyymmdd(dates):
-    """Convert date str from (YY)YYMMDD to YYYYMMDD format"""
+    """Convert date str from (YY)YYMMDD(THHMM) to YYYYMMDD(THHMM) format"""
     if isinstance(dates, str):
-        if len(dates) == 6:
+        if len(dates.split('T')[0]) == 6:
             datesOut = yymmdd2yyyymmdd(dates)
         else:
             datesOut = dates
@@ -187,7 +187,7 @@ def yyyymmdd(dates):
     elif isinstance(dates, list):
         datesOut = []
         for date in dates:
-            if len(date) == 6:
+            if len(date.split('T')[0]) == 6:
                 date = yymmdd2yyyymmdd(date)
             datesOut.append(date)
 
@@ -290,7 +290,7 @@ def date_list2tbase(date_list):
     # date str to dt object
     date_list = yyyymmdd(date_list)
     date_format = get_date_str_format(str(date_list))
-    dates = [dt.strptime(i, date_format) for i in date_list]
+    dates = [dt.datetime.strptime(i, date_format) for i in date_list]
 
     # dt object to time difference in days
     tbase = []
@@ -315,16 +315,61 @@ def date_list2vector(date_list):
     """
     date_list = yyyymmdd(date_list)
     date_format = get_date_str_format(str(date_list))
-    dates = [dt.strptime(i, date_format) for i in date_list]
+    dates = [dt.datetime.strptime(i, date_format) for i in date_list]
 
     # date in year - float format
     datevector = []
     for d in dates:
-        date_vec = (d.year + (d.timetuple().tm_yday - 1) / 365.25 + 
-                    d.hour / (365.25 * 24) + 
-                    d.minute / (365.25 * 24 * 60) + 
+        date_vec = (d.year + (d.timetuple().tm_yday - 1) / 365.25 +
+                    d.hour / (365.25 * 24) +
+                    d.minute / (365.25 * 24 * 60) +
                     d.second / (365.25 * 24 * 60 * 60))
         datevector.append(date_vec)
 
     return dates, datevector
 
+
+################################################################
+def get_date_range(dmin, dmax):
+    """Make a list of dates with one-day interval [dmin, dmax]
+    Parameters: dmin      : str in YYYYMMDD format
+                dmax      : str in YYYYMMDD format
+    Returns:    date_list : list of str in YYYYMMDD format
+    """
+    t1 = '{}-{}-{}'.format(dmin[:4], dmin[4:6], dmin[6:])
+    t2 = '{}-{}-{}'.format(dmax[:4], dmax[4:6], dmax[6:])
+    date_objs = np.arange(t1, t2, dtype='datetime64[D]').astype('M8[D]').astype('O')
+    date_list = [obj.strftime('%Y%m%d') for obj in date_objs] + [dmax]
+    return date_list
+
+
+def utc2solar_time(utc_time, longitude):
+    """Convert UTC time to solar local time.
+    Solar time: https://en.wikipedia.org/wiki/Solar_time
+    Link: https://stackoverflow.com/questions/13314626
+
+    Parameters: utc_time   - datetime.datetime object for the UTC time
+                longitude  - float, longitude of the observer in degrees
+    Returns:    solar_time - datetime.datetime object for the local solar time
+    Example:    utc_time = dt.datetime(2015, 2, 9, 3, 18, 48)
+                solar_time = ptime.utc2solar_time(utc_time, 130.7)
+    """
+    from math import pi, cos, sin
+    # use 366 for leap years
+    if utc_time.year % 4 == 0 and utc_time.year % 100 != 0 and utc_time.year % 400 != 0:
+        year_len = 366
+    else:
+        year_len = 365
+
+    gamma = 2 * pi / year_len * (utc_time.timetuple().tm_yday - 1 + float(utc_time.hour - 12) / 24)
+    eqtime = 229.18 * (0.000075 + 0.001868 * cos(gamma) - 0.032077 * sin(gamma) \
+             - 0.014615 * cos(2 * gamma) - 0.040849 * sin(2 * gamma))
+    #decl = 0.006918 - 0.399912 * cos(gamma) + 0.070257 * sin(gamma) \
+    #       - 0.006758 * cos(2 * gamma) + 0.000907 * sin(2 * gamma) \
+    #       - 0.002697 * cos(3 * gamma) + 0.00148 * sin(3 * gamma)
+    time_offset = eqtime + 4 * longitude
+    tst = utc_time.hour * 60 + utc_time.minute + utc_time.second / 60 + time_offset
+    solar_time = dt.datetime.combine(utc_time.date(), dt.time(0)) + dt.timedelta(minutes=tst)
+
+    return solar_time
+
