@@ -156,7 +156,7 @@ def nonzero_mask(File, out_file='maskConnComp.h5', datasetName=None):
 
 
 def spatial_average(File, datasetName='coherence', maskFile=None, box=None,
-                    saveList=False, checkAoi=True):
+                    saveList=False, checkAoi=True, reverseMask=False, threshold=None):
     """Read/Calculate Spatial Average of input file.
 
     If input file is text file, read it directly;
@@ -165,15 +165,17 @@ def spatial_average(File, datasetName='coherence', maskFile=None, box=None,
         Otherwise, calculate it from data file.
 
         Only non-nan pixel is considered.
-    Parameters: File     : string, path of input file
-                maskFile : string, path of mask file, e.g. maskTempCoh.h5
-                box      : 4-tuple defining the left, upper, right, and lower pixel coordinate
-                saveList : bool, save (list of) mean value into text file
-    Returns:    meanList : list for float, average value in space for each epoch of input file
-                dateList : list of string for date info
-                    date12_list, e.g. 101120-110220, for interferograms/coherence
-                    date8_list, e.g. 20101120, for timeseries
-                    file name, e.g. velocity.h5, for all the other file types
+    Parameters: File        - string, path of input file
+                maskFile    - string, path of mask file, e.g. maskTempCoh.h5
+                box         - 4-tuple defining the left, upper, right, and lower pixel coordinate
+                saveList    - bool, save (list of) mean value into text file
+                reverseMask - bool, perform analysis within masked regions instead of outside of them
+                threshold   - float, calculate area ratio above threshold instead of spatial average
+    Returns:    meanList    - list for float, average value in space for each epoch of input file
+                dateList    - list of string for date info
+                              date12_list, e.g. 101120-110220, for interferograms/coherence
+                              date8_list, e.g. 20101120, for timeseries
+                              file name, e.g. velocity.h5, for all the other file types
     Example:    meanList = spatial_average('inputs/ifgramStack.h5')[0]
                 meanList, date12_list = spatial_average('inputs/ifgramStack.h5',
                                                         maskFile='maskTempCoh.h5',
@@ -192,12 +194,10 @@ def spatial_average(File, datasetName='coherence', maskFile=None, box=None,
         box = (0, 0, int(atr['WIDTH']), int(atr['LENGTH']))
 
     # default output filename
-    if k == 'ifgramStack':
-        prefix = datasetName
-    else:
-        prefix = os.path.splitext(os.path.basename(File))[0]
-    suffix = 'SpatialAvg.txt'
-    txtFile = prefix + suffix
+    prefix = datasetName if k == 'ifgramStack' else os.path.splitext(os.path.basename(File))[0]
+    suffix = 'SpatialAvg' if threshold is None else 'AreaRatio'
+    suffix += 'RevMsk' if reverseMask else ''
+    txtFile = prefix + suffix + '.txt'
 
     # If input is text file
     if File.endswith(suffix):
@@ -206,9 +206,11 @@ def spatial_average(File, datasetName='coherence', maskFile=None, box=None,
         return meanList, dateList
 
     # Read existing txt file only if 1) data file is older AND 2) same AOI
-    file_line = '# Data file: {}\n'.format(os.path.basename(File))
-    mask_line = '# Mask file: {}\n'.format(maskFile)
-    aoi_line  = '# AOI box: {}\n'.format(box)
+    file_line  = '# Data file: {}\n'.format(os.path.basename(File))
+    mask_line  = '# Mask file: {}\n'.format(maskFile)
+    aoi_line   = '# AOI box: {}\n'.format(box)
+    thres_line = '# Threshold: {}\n'.format(threshold)
+
     try:
         # Read AOI line from existing txt file
         fl = open(txtFile, 'r')
@@ -242,26 +244,37 @@ def spatial_average(File, datasetName='coherence', maskFile=None, box=None,
     else:
         useMedian = False
 
-    # Calculate mean coherence list
+    # Calculate mean coherence or area ratio list
     if k == 'ifgramStack':
         obj = ifgramStack(File)
         obj.open(print_msg=False)
         meanList, dateList = obj.spatial_average(datasetName=datasetName,
                                                  maskFile=maskFile,
                                                  box=box,
-                                                 useMedian=useMedian)
+                                                 useMedian=useMedian,
+                                                 reverseMask=reverseMask,
+                                                 threshold=threshold)
         pbase = obj.pbaseIfgram
         tbase = obj.tbaseIfgram
         obj.close()
     elif k == 'timeseries':
         meanList, dateList = timeseries(File).spatial_average(maskFile=maskFile,
-                                                              box=box)
+                                                              box=box,
+                                                              reverseMask=reverseMask,
+                                                              threshold=threshold)
     else:
         data = readfile.read(File, box=box)[0]
         if maskFile and os.path.isfile(maskFile):
             print('mask from file: '+maskFile)
             mask = readfile.read(maskFile, datasetName='mask', box=box)[0]
-            data[mask == 0.] = np.nan
+            data[mask == int(reverseMask)] = np.nan
+
+        # calculate area ratio if threshold is specified
+        # percentage of pixels with value above the threshold
+        if threshold is not None:
+            data[data > threshold] = 1
+            data[data <= threshold] = 0
+
         meanList = np.nanmean(data)
         dateList = [os.path.basename(File)]
 
@@ -270,7 +283,7 @@ def spatial_average(File, datasetName='coherence', maskFile=None, box=None,
         print('write average value in space into text file: '+txtFile)
         fl = open(txtFile, 'w')
         # Write comments
-        fl.write(file_line+mask_line+aoi_line)
+        fl.write(file_line+mask_line+aoi_line+thres_line)
         # Write data list
         numLine = len(dateList)
         if k == 'ifgramStack':
@@ -505,6 +518,7 @@ def update_template_file(template_file, extra_dict, delimiter='='):
         if not line.startswith(('%', '#')) and len(c) > 1:
             key = c[0]
             value = str.replace(c[1], '\n', '').split("#")[0].strip()
+            value = value.replace('*', '\*')  # interpret * as character
             if key in extra_dict.keys() and extra_dict[key] != value:
                 # use "= {OLD_VALUE}" for search/replace to be more robust
                 # against the scenario when key name contains {OLD_VALUE}
@@ -705,13 +719,14 @@ def check_template_auto_value(templateDict, auto_file='defaults/smallbaselineApp
     return templateDict
 
 
-def run_deramp(fname, ramp_type, mask_file=None, out_file=None, datasetName=None):
+def run_deramp(fname, ramp_type, mask_file=None, out_file=None, datasetName=None, save_ramp_coeff=False):
     """ Remove ramp from each 2D matrix of input file
     Parameters: fname     : str, data file to be derampped
                 ramp_type : str, name of ramp to be estimated.
                 mask_file : str, file of mask of pixels used for ramp estimation
                 out_file  : str, output file name
-                datasetName : str, output dataset name, for ifgramStack file type only
+                datasetName     : str, output dataset name, for ifgramStack file type only
+                save_ramp_coeff : bool, save the estimated ramp coefficients to text file
     Returns:    out_file  : str, output file name
     """
     start_time = time.time()
@@ -735,6 +750,16 @@ def run_deramp(fname, ramp_type, mask_file=None, out_file=None, datasetName=None
         mask = np.ones((length, width), dtype=np.bool_)
         print('use mask of the whole area')
 
+    # write coefficient of specified surface function fit
+    coeff_file = None
+    if save_ramp_coeff:
+        fbase = os.path.splitext(os.path.basename(fname))[0]
+        coeff_file = os.path.join(os.path.dirname(fbase), 'rampCoeff_{}.txt'.format(fbase))
+        with open(coeff_file, 'w') as f:
+            f.write('# input  file: {}\n'.format(fname))
+            f.write('# output file: {}\n'.format(out_file))
+            f.write('# ramp type: {}\n'.format(ramp_type))
+
     # deramping
     if k == 'timeseries':
         # write HDF5 file with defined metadata and (empty) dataset structure
@@ -745,10 +770,17 @@ def run_deramp(fname, ramp_type, mask_file=None, out_file=None, datasetName=None
         num_date = len(date_list)
         prog_bar = ptime.progressBar(maxValue=num_date)
         for i in range(num_date):
+            if coeff_file:
+                # prepend epoch name to line of coefficients
+                with open(coeff_file, 'a') as f:
+                    f.write('{}    '.format((date_list[i])))
             # read
             data = readfile.read(fname, datasetName=date_list[i])[0]
             # deramp
-            data = deramp(data, mask, ramp_type=ramp_type, metadata=atr)[0]
+            data = deramp(data, mask,
+                          ramp_type=ramp_type,
+                          metadata=atr,
+                          coeff_file=coeff_file)[0]
             # write
             writefile.write_hdf5_block(out_file, data,
                                        datasetName='timeseries',
@@ -763,6 +795,7 @@ def run_deramp(fname, ramp_type, mask_file=None, out_file=None, datasetName=None
         obj.open(print_msg=False)
         if not datasetName:
             datasetName = 'unwrapPhase'
+
         with h5py.File(fname, 'a') as f:
             ds = f[datasetName]
             dsNameOut = '{}_ramp'.format(datasetName)
@@ -779,8 +812,18 @@ def run_deramp(fname, ramp_type, mask_file=None, out_file=None, datasetName=None
 
             prog_bar = ptime.progressBar(maxValue=obj.numIfgram)
             for i in range(obj.numIfgram):
+                if coeff_file:
+                    # prepend IFG date12 to line of coefficients
+                    with open(coeff_file, 'a') as f:
+                        f.write('{}    '.format(str(obj.date12List[i])))
+                # read
                 data = ds[i, :, :]
-                data = deramp(data, mask, ramp_type=ramp_type, metadata=atr)[0]
+                # deramp
+                data = deramp(data, mask,
+                              ramp_type=ramp_type,
+                              metadata=atr,
+                              coeff_file=coeff_file)[0]
+                # write
                 dsOut[i, :, :] = data
                 prog_bar.update(i+1, suffix='{}/{}'.format(i+1, obj.numIfgram))
             prog_bar.close()
@@ -788,8 +831,18 @@ def run_deramp(fname, ramp_type, mask_file=None, out_file=None, datasetName=None
 
     # Single Dataset File
     else:
+        if coeff_file:
+            # prepend file-type to line of coefficients
+            with open(coeff_file, 'a') as f:
+                f.write('{}    '.format(atr['FILE_TYPE']))
+        # read
         data = readfile.read(fname)[0]
-        data = deramp(data, mask, ramp_type, metadata=atr)[0]
+        # deramp
+        data = deramp(data, mask,
+                      ramp_type=ramp_type,
+                      metadata=atr,
+                      coeff_file=coeff_file)[0]
+        # write
         print('writing >>> {}'.format(out_file))
         writefile.write(data, out_file=out_file, ref_file=fname)
 
